#include <led-matrix-32-16.hpp>
#include <Arduino.h>
#include <WiFi.h>
#include <NTPClient.h>
#include <WiFiUdp.h>

LedMetrix32x16 ledMatrix; // Create an instance of the LedMetrix32x16 class

#define YOUR_WIFI_SSID "WiFi - Mekatronika"
#define YOUR_WIFI_PASSWD "tanyapakimron"
#define SHOW_TIME_PERIOD 1000

// Reduce memory usage by using static allocation
static MatrixPanel_I2S_DMA *display = nullptr;
static CustomPxBasePanel *virtualDisplay = nullptr;

// NTP Client setup
WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP, "pool.ntp.org", 25200); // UTC+7 for Jakarta

void setup() {
  Serial.begin(115200); // Initialize serial communication for debugging
  delay(1000); // Give time for system to stabilize
  
  // Initialize WiFi with optimized settings
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(1000);
  
  // Optimize WiFi power and performance
  WiFi.setTxPower(WIFI_POWER_19_5dBm);
  WiFi.setSleep(false); // Disable WiFi sleep mode
  
  // Connect to WiFi
  Serial.println("Connecting to WiFi...");
  Serial.print("SSID: ");
  Serial.println(YOUR_WIFI_SSID);
  
  WiFi.begin(YOUR_WIFI_SSID, YOUR_WIFI_PASSWD);
  
  // Wait for connection with timeout
  int timeout = 0;
  while (WiFi.status() != WL_CONNECTED && timeout < 20) {
    delay(500);
    Serial.print(".");
    Serial.print("Status: ");
    switch(WiFi.status()) {
      case WL_IDLE_STATUS:
        Serial.println("WL_IDLE_STATUS");
        break;
      case WL_NO_SSID_AVAIL:
        Serial.println("WL_NO_SSID_AVAIL");
        break;
      case WL_CONNECT_FAILED:
        Serial.println("WL_CONNECT_FAILED");
        break;
      case WL_CONNECTION_LOST:
        Serial.println("WL_CONNECTION_LOST");
        break;
      case WL_DISCONNECTED:
        Serial.println("WL_DISCONNECTED");
        break;
      default:
        Serial.println("Unknown status");
    }
    timeout++;
  }
  
  if (WiFi.status() != WL_CONNECTED) {
    Serial.println("");
    Serial.println("WiFi connection failed!");
    Serial.println("Please check:");
    Serial.println("1. WiFi SSID and password are correct");
    Serial.println("2. WiFi router is in range");
    Serial.println("3. WiFi router is working properly");
    while (true) {
      delay(1000);
      Serial.print("Retrying connection... Status: ");
      Serial.println(WiFi.status());
      WiFi.begin(YOUR_WIFI_SSID, YOUR_WIFI_PASSWD);
    }
  }

  Serial.println("");
  Serial.println("WiFi connected!");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.print("Signal strength (RSSI): ");
  Serial.println(WiFi.RSSI());
  
  // Initialize LED matrix after WiFi is connected
  ledMatrix.begin(1, 3);
  display = ledMatrix.getDisplay();
  virtualDisplay = ledMatrix.getVirtualDisplay();
  display->setBrightness(50);

  // Initialize NTP
  timeClient.begin();
  timeClient.update();
}

int last = 0;
void loop() {
  // Use static display pointers to reduce memory allocation
  if (!display || !virtualDisplay) 
    while(1);
  
  // Update time every minute
  if (millis() - last >= 60000) {
    timeClient.update();
    last = millis();
  }

  // Get current time
  String timeString = timeClient.getFormattedTime();
  
  // Display time
  virtualDisplay->clearScreen();
  virtualDisplay->setTextSize(2);
  virtualDisplay->setTextColor(virtualDisplay->color565(255, 255, 255));
  virtualDisplay->setCursor(0, 0);
  virtualDisplay->println(timeString);
  delay(3000);

  // draw a red pixel at each position on the display
  for (int i = 0; i < virtualDisplay->height(); i++)
  {
    for (int j = 0; j < virtualDisplay->width(); j++)
    {
      virtualDisplay->drawPixel(j, i, virtualDisplay->color565(255, 0, 0)); // Draw a red pixel at (j, i)
      delay(10); // Delay to visualize the drawing
    }
  }
  delay(1000); // Wait for a second before starting over

  // Gradually turn brightness from Bright to Dim 
  for (int i = 255; i >= 0; i--)
  {
    display->setBrightness(i); // Gradually decrease brightness
    delay(10); // Delay to visualize the brightness change
  }
  
  // Gradually turn brightness from  Dim to Bright
  for (int i = 0; i <= 255; i++)
  {
    display->setBrightness(i); // Gradually increase brightness
    delay(10); // Delay to visualize the brightness change
  }
  delay(1000); // Wait for a second before starting over


  virtualDisplay->clearScreen(); // Clear the virtual display
  virtualDisplay->setTextSize(2); // Set text size to 2
  virtualDisplay->setTextColor(virtualDisplay->color565(255, 255, 255)); // Set text color to white
  for (int i = 0; i >= (-9 * 13); i--)
  {
    virtualDisplay->setCursor(i,0);          // Set cursor to the top-left corner
    virtualDisplay->println("Hello, World!"); // Print "Hello, World!" on the display
    delay(100);
    virtualDisplay->clearScreen(); // Clear the virtual display after each print
  }

  uint16_t color [] = { 
    virtualDisplay->color565(255, 0, 0),   // Red
    virtualDisplay->color565(0, 255, 0),   // Green
    virtualDisplay->color565(0, 0, 255),   // Blue
    virtualDisplay->color565(255, 255, 0), // Yellow
    virtualDisplay->color565(255, 0, 255), // Magenta
    virtualDisplay->color565(0, 255, 255)  // Cyan
  };

  virtualDisplay->clearScreen();  // Clear the virtual display
  virtualDisplay->setTextSize(1); // Set text size to 2
  for (int i = 0; i < sizeof(color) / sizeof(color[0]); i++)
  {
    virtualDisplay->setCursor(0, 0);          // Set cursor to the top-left corner
    virtualDisplay->setTextColor(color[i]);   // Set text color to white
    virtualDisplay->println("Hello, World!"); // Print "Hello, World!" on the display
    delay(3000);                              // Wait for 2 seconds to visualize the text
  }
}
